chrome.action.onClicked.addListener(function(tab) {
    chrome.tabs.query({ active: true, currentWindow: true }, function(tabs) {
        chrome.tabs.sendMessage(tabs[0].id, { greeting: "NEplugin" }, function(response) {
            console.log('content script response');
        });
    });
})

import { http } from './lib/http.js';
import { NEUtils, Base64 } from './lib/common.js';

var downloadItemID = -1;

chrome.runtime.onMessage.addListener(
    (message, sender, sendResponse) => {
        console.log(sender.tab ?
            "from a content script:" + sender.tab.url :
            "from the extension");

        new Promise(async (resolve, reject) => {
            if (typeof message != 'object' || !message.type) {
                console.error("消息格式不符合规范：", message);
                reject(`消息 ${JSON.stringify(message)} 格式不符合规范。`);
                return;
            }
            switch (message.type) {
                case 'checkNEOpened':
                    {
                        http.get({ url: "http://127.0.0.1:49527/GetReference" }).then(response => {
                            resolve(NEUtils.parseHttpResponse(response));
                        }).catch(error => {
                            reject(NEUtils.parseHttpResponse(error));
                        });
                    }
                    break;
                case 'saveToNE':
                    {
                        http.post({
                            url: 'http://127.0.0.1:49527/SaveRefs',
                            param: { 'Refs': message.data.data, 'targetdir': message.data.dir }
                        }).then(response => {
                            resolve(NEUtils.parseHttpResponse(response));
                            if (response.Status == '失败') {
                                sendResponse({ requestType: '0', msg: response.msg });
                            } else {
                                sendResponse({ requestType: '1' });
                            }
                        }).catch(error => {
                            sendResponse({ requestType: '0', msg: '请确保NoteExpress客户端正在运行，再点击保存' });
                            // reject(NEUtils.parseHttpResponse(error));
                        });
                    }
                    break;
                case 'saveFullTextToNE':
                    {
                        var jsonData = JSON.parse(message.data.data);
                        jsonData = jsonData.Refs;
                        jsonData = jsonData[0];
                        jsonData = Base64.decode(jsonData);

                        let downloadStartTimer = null;
                        let downloadCompleteTimer = null;
                        let downloadItemID = null;

                        const startTimeout = setTimeout(() => {
                            handleTimeout('下载未开始');
                        }, 5000);

                        chrome.downloads.onCreated.addListener(function onCreateCallBack(DownloadItem) {
                            if (DownloadItem.url = message.data.pdfPath) {
                                clearTimeout(startTimeout); // 清除开始超时
                                downloadItemID = DownloadItem.id;
                                downloadCompleteTimer = setTimeout(() => {
                                    handleTimeout('下载未完成');
                                }, 30000);

                                console.log("全文下载项目ID：" + downloadItemID);
                                chrome.downloads.onCreated.removeListener(onCreateCallBack);
                            }
                        });

                        function handleTimeout(reason) {
                            // 无论是否在监听都执行保存操作
                            http.post({
                                url: 'http://127.0.0.1:49527/SaveRefs',
                                param: { 
                                    'Refs': message.data.data, 
                                    'targetdir': message.data.dir,
                                    'timeoutReason': reason // 添加超时原因参数
                                }
                            }).then(response => {
                                sendResponse({ 
                                    requestType: '0', 
                                    msg: response.msg + reason,
                                    downmsg: '下载超时：' + reason
                                });
                            }).catch(error => {
                                sendResponse({ 
                                    requestType: '0', 
                                    msg: '下载超时且保存失败：' + reason,
                                    downmsg: '下载超时'
                                });
                            }).finally(() => {
                                // 清理所有监听和定时器
                                chrome.downloads.onCreated.removeListener(onCreateCallBack);
                                chrome.downloads.onChanged.removeListener(changeFunc);
                                if (downloadCompleteTimer) clearTimeout(downloadCompleteTimer);
                            });
                        }




                        chrome.downloads.onChanged.addListener(function changeFunc(obj) {
                            console.log('downloads is listining');
                            if (obj.hasOwnProperty("state") && obj.state.current === "complete" && (downloadItemID = obj.id)) {
                                clearTimeout(downloadCompleteTimer); // 清除完成超时
                                http.post({
                                    url: 'http://127.0.0.1:49527/SaveRefs',
                                    param: { 'Refs': message.data.data, 'targetdir': message.data.dir }
                                }).then(response => {
                                    resolve(NEUtils.parseHttpResponse(response));
                                    if (response.Status == '失败') {
                                        sendResponse({ requestType: '0', msg: response.msg });
                                    } else {
                                        chrome.downloads.search({ id: obj.id }, function(items) {
                                            var filename = items[0].filename;
                                            var option = { url: 'http://127.0.0.1:49527/AttachFile' + '?UUID=' + (JSON.parse(jsonData)).uuid + '&FileName=' + encodeURIComponent(filename) };
                                            console.log(option);
                                            http.get(option).then(response => {
                                                chrome.downloads.onChanged.removeListener(changeFunc)
                                                resolve(NEUtils.parseHttpResponse(response));
                                            }).catch(error => {
                                                chrome.downloads.onChanged.removeListener(changeFunc)
                                                reject(NEUtils.parseHttpResponse(error));
                                            });
                                        });
                                        sendResponse({ requestType: '1' });
                                    }
                                }).catch(error => {
                                    sendResponse({ requestType: '0', msg: '请确保NoteExpress客户端正在运行，再点击保存' });
                                    // reject(NEUtils.parseHttpResponse(error));
                                });
                            }
                        });
                    }
                    break;
                case 'crossUrl':
                    {
                        http.get({ url: message.data.targetUrl }).then(response => {
                            resolve(NEUtils.parseHttpResponse(response));
                        }).catch(error => {
                            reject(NEUtils.parseHttpResponse(error));
                        });
                    }
                    break;
                default:
                    {
                        console.log('wrong request string!');
                        reject(`消息 ${message} 类型非法。`);
                        break;
                    }
            }
        }).then((response) => {
            sendResponse(NEUtils.packMsgRep(true, response, message));
            console.log(`消息 ${JSON.stringify(message)} 处理完成。`);
        }).catch(e => {
            switch (message.type) {
                case 'checkNEOpened':
                    {
                        sendResponse(NEUtils.packMsgRep(false, e, message));
                        console.warn(`NE尚未打开；消息 ${JSON.stringify(message)} 处理失败：`, e);
                        break;
                    }
                default:
                    {
                        sendResponse(NEUtils.packMsgRep(false, e, message));
                        console.error(`消息 ${JSON.stringify(message)} 处理失败：`, e);
                    }
            }
        });
        return true;
    }
);
console.log("background.js is loadded");