function startup() {
	// 显示加载页
	showLoading();
	
	// 获取导出表单数据
	let exportForm = document.getElementById('exportArticles');
	if (!exportForm) {
		console.log("Export form not found");
		return;
	}

	// 构建FormData对象
	let formData = new FormData(exportForm);
	
	// 发送POST请求获取引文数据
	fetch('/export', {
		method: 'POST',
		body: formData,
		headers: {
			'Accept': 'application/json'
		}
	})
	.then(response => response.json())
	.then(data => {
		// 解析返回的引文数据
		let citation = parseMDPICitation(data);
		// 显示引文信息
		ParserDetailsJson([citation]);
	})
	.catch(error => {
		console.error('Error fetching citation:', error);
		hideLoading();
	});
}

function parseMDPICitation(data) {
	// 构建引文对象
	let citation = {
		"Reference Type": "Journal Article",
		"Title": document.querySelector('h1.title')?.textContent?.trim(),
		"Author": getAuthors(),
		"Year": getPublicationYear(),
		"Journal": document.querySelector('span[itemprop="name"]')?.textContent?.trim(),
		"Volume": document.querySelector('span[itemprop="volumeNumber"]')?.textContent?.trim(),
		"Issue": document.querySelector('span[itemprop="issueNumber"]')?.textContent?.trim(),
		"Pages": document.querySelector('span[itemprop="pageStart"]')?.textContent?.trim(),
		"Abstract": document.querySelector('div.art-abstract')?.textContent?.trim(),
		"Keywords": getKeywords(),
		"DOI": document.querySelector('span[itemprop="doi"]')?.textContent?.trim()
	};

	return citation;
}

function getAuthors() {
	let authors = [];
	document.querySelectorAll('span[itemprop="author"] span[itemprop="name"]').forEach(author => {
		authors.push(author.textContent.trim());
	});
	return authors.join(';');
}

function getPublicationYear() {
	let date = document.querySelector('time[itemprop="datePublished"]')?.getAttribute('datetime');
	return date ? new Date(date).getFullYear().toString() : '';
}

function getKeywords() {
	let keywords = [];
	document.querySelectorAll('span.keyword').forEach(keyword => {
		keywords.push(keyword.textContent.trim());
	});
	return keywords.join(';');
}